import numpy as np
import matplotlib.pyplot as plt
from astropy.io import fits as pyfits
from PyAstronomy import pyasl
import scipy
from scipy import interpolate
from scipy import ndimage
from scipy import signal
import pickle
from matplotlib.backends.backend_pdf import PdfPages
import os
#from pyevolve import G1DList
#from pyevolve import GSimpleGA

from multiprocessing import Pool
import time

def download_models(webpage='http://svo2.cab.inta-csic.es/theory/models/coelho/high/data/',dest='../../data/'):

	os.system('mkdir '+dest+'/COELHO2014')
	cwd = os.getcwd()
	os.chdir(dest+'/COELHO2014')
	tf = np.arange(6000,10001,250)
	gf = np.arange(2.5,4.6,0.5)
	#gf = np.array([2.5])
	zf = np.array([-1.,-0.5,0.0,0.2])

	for t in tf:
		for g in gf:
			for z in zf:
				modname = get_modname(t,g,z)
				if z<0:
					sz = 'm'
				else:
					sz = 'p'
				sz = sz+str(float(np.absolute(z))).replace('.','')+'p00/'
				os.system('wget ' + webpage+sz+modname+'.fits')
				os.system('wget ' + webpage+sz+modname+'plc.fits')
	os.chdir(cwd)

	return True

def n_Edlen(l):
    sigma = 1e4 / l
    sigma2 = sigma*sigma
    n = 1 + 1e-8 * (8342.13 + 2406030 / (130-sigma2) + 15997/(38.9-sigma2))
    return n

def n_Morton(l):
    sigma = 1e4 / l
    sigma2 = sigma*sigma
    n = 1 + 6.4328e-5 + 2.94981e-2 / (146.-sigma2) + 2.5540e-4/(41.-sigma2)
    return n

def ToAir(l):
    return (l / n_Edlen(l))

def ToVacuum(l):
    cond = 1
    l_prev = l.copy()
    while(cond):
        l_new = n_Edlen(l_prev) * l
        if (max(np.absolute(l_new - l_prev)) < 1e-10): cond = 0
        l_prev = l_new
    return l_prev

def get_modname(t,g,z):
	st = str(int(t))
	if t<10000:
		st = '0'+st
	sg = '+'+str(np.around(g,1))
	if z < 0:
		sz = 'm'
	else:
		sz = 'p'
	z=float(z)
	sz = sz + str(np.around(np.absolute(z),1))
	sz = sz.replace('.','')
	return 't'+st+'_g'+sg+'_'+sz+'p00_hr'

def get_model(t,g,z,model_path='../../data/COELHO2014/'):
	modname = model_path + get_modname(t,g,z)
	try:
		out = pyfits.getdata(modname+'.fits')
	except:
		out = pyfits.getdata(modname+'plc.fits')
	return out

def get_near(x,vec):

	if x == vec[0]:
		mmin = vec[0]
		mmax = vec[1]
	elif x == vec[-1]:
		mmin = vec[-2]
		mmax = vec[-1]
	else:
		tvec = vec - x
		In  = np.where(tvec < 0)[0]
		mmin = tvec[In].max() + x
		Ix = np.where(tvec >= 0)[0]
		mmax = tvec[Ix].min() + x
	return mmin,mmax

def trilinear_interpolation(t,g,z,model_path='../../data/COELHO2014/'):
	teffs = np.arange(6000,10001,250)
	loggs = np.arange(2.5,4.6,0.5)
	fehs  = np.array([-1.,-0.5,0.0,0.2])
	x0,x1 = get_near(t,teffs)
	y0,y1 = get_near(g,loggs)
	z0,z1 = get_near(z,fehs)
	xd = (t-x0)/(x1-x0)
	yd = (g-y0)/(y1-y0)
	zd = (z-z0)/(z1-z0)

	try:
		hd = pyfits.getheader(model_path+get_modname(x0,y0,z0)+'.fits')
	except:
		hd = pyfits.getheader(model_path+get_modname(x0,y0,z0)+'plc.fits')

	c000 = get_model(x0,y0,z0,model_path)
	c001 = get_model(x0,y0,z1,model_path)
	c010 = get_model(x0,y1,z0,model_path)
	c100 = get_model(x1,y0,z0,model_path)
	c110 = get_model(x1,y1,z0,model_path)
	c101 = get_model(x1,y0,z1,model_path)
	c011 = get_model(x0,y1,z1,model_path)
	c111 = get_model(x1,y1,z1,model_path)

	wav = np.arange(len(c111))*hd['CDELT1'] + hd['CRVAL1']

	c00 = c000*(1-xd) + c100*xd
	c01 = c001*(1-xd) + c101*xd
	c10 = c010*(1-xd) + c110*xd
	c11 = c011*(1-xd) + c111*xd

	c0 = c00*(1-yd) + c10*yd
	c1 = c01*(1-yd) + c11*yd

	c = c0*(1-zd) + c1*zd

	return wav,c

def normalize_model(w,f):
	ow = w.copy()
	of = f.copy()
	#plot(w,f)
	while True:
		#medflts = scipy.signal.medfilt(f,1001)
		coef = np.polyfit(w,f,6)
		fited = np.polyval(coef,w)
		res = f - fited
		I = np.where(res > -np.sqrt(np.var(res)))[0]
		w,f = w[I],f[I]
		if len(w) < 0.3* len(ow):
			break
	#plot(ow,np.polyval(coef,ow))
	#show()
	return coef

def spec_ccf(sw,sf,mw,mf,vi,vf,dv):
	mf = mf -1
	mf = -mf
	#plot(mw,mf)
	tck = interpolate.splrep(mw,mf,k=1)
	v = vi
	retccf = []
	vels = []
	while v<=vf:
		swt = sw * (1 + v/299792.458)
		mft = interpolate.splev(swt,tck)
		#if v == 0:
		#	plot(swt,mft)
		#	plot(swt,sft)
		#	show()
		mft -= np.mean(mft)
		sft = sf - np.mean(sf)
		#sft = sf.copy()
		#print(np.sum(mft**2),np.sum(sft**2))
		retccf.append(np.sum(mft*sft)/np.sqrt(np.sum(mft**2)*np.sum(sft**2)))
		vels.append(v)
		v+=dv

	return np.array(vels),np.array(retccf)

def ccf_fft(swt,sft,mwt,mft):
	mf = mft -1
	mf = -mf

	#plot(mw,mf)
	tck = interpolate.splrep(np.log(mwt),mf,k=1)

	sw = np.log(swt)
	tck2 = interpolate.splrep(sw,sft,k=1)

	nsw = np.linspace(sw[0], sw[-1], 5000)
	sf  =  interpolate.splev(nsw,tck2)

	mf  =  interpolate.splev(nsw,tck)

	sf -= np.mean(sf)
	mf -= np.mean(mf)
	plt.plot(nsw,sf)
	plt.plot(nsw,mf)
	plt.show()
	retccf = np.fft.ifft(np.conj(np.fft.fft(sf))*np.fft.fft(mf))
	retccf = np.hstack((retccf[2500:],retccf[:2500]))
	retvels = np.arange(len(retccf)) - 0.5*len(retccf)
	retvels *= (nsw[1]-nsw[0])
	retvels = 299792.458*(np.exp(retvels)-1.)
	return retvels, retccf

def ccf_simple(sw,sf,mw,mf,rv):
	mf = mf -1
	mf = -mf
	#plot(mw,mf)
	tck = interpolate.splrep(mw,mf,k=1)
	swt = sw * (1 + rv/299792.458)
	mft = interpolate.splev(swt,tck)
	mft -= np.mean(mft)
	sft = sf - np.mean(sf)
	return np.sum(mft*sft)/np.sqrt(np.sum(mft**2)*np.sum(sft**2))

def clean_strong_lines(mw,sc,mode=1):
	if mode==1:
		#""""
		I = np.where((mw>6520)&(mw<6600))[0]
		sc[I] = 1.
		I = np.where((mw>5888)&(mw<5897))[0]
		sc[I] = 1.
		I = np.where((mw>4310)&(mw<4360))[0]
		sc[I] = 1.
		I = np.where((mw>4840)&(mw<4880))[0]
		sc[I] = 1.
		I = np.where((mw>4070)&(mw<4130))[0]
		sc[I] = 1.
		I = np.where((mw>3875)&(mw<3900))[0]
		sc[I] = 1.
		I = np.where((mw>3920)&(mw<3945))[0]
		sc[I] = 1.
		I = np.where((mw>3955)&(mw<3980))[0]
		sc[I] = 1.
		I = np.where(mw<3850)[0]
		sc[I] = 1.
		#"""
	if mode==2:
		#""""
		I = np.where((mw>6550)&(mw<6570))[0]
		sc[I] = 1.
		I = np.where((mw>5888)&(mw<5897))[0]
		sc[I] = 1.
		I = np.where((mw>4320)&(mw<4350))[0]
		sc[I] = 1.
		I = np.where((mw>4850)&(mw<4870))[0]
		sc[I] = 1.
		I = np.where((mw>4090)&(mw<4110))[0]
		sc[I] = 1.
		I = np.where((mw>3875)&(mw<3900))[0]
		sc[I] = 1.
		I = np.where((mw>3920)&(mw<3945))[0]
		sc[I] = 1.
		I = np.where((mw>3955)&(mw<3980))[0]
		sc[I] = 1.
		I = np.where(mw<3850)[0]
		sc[I] = 1.
		#"""
	return sc

def RVforFR(wavs,flxs,teff=6700,logg=4.0,feh=-1.0,vsini=100.,model_path='../../data/COELHO2014/',vmin=-1000.,vmax=1000.,vstep=10.):
	def fitfunc(p,x):
		ret = p[3] + p[0] * np.exp(-.5*((x-p[1])/p[2])**2)
		return ret
	errfunc = lambda p,x,y: np.ravel( (fitfunc(p,x)-y) )

	#sc = get_model(teff,logg,feh)
	#hd = pyfits.getheader(model_path+get_modname(7000,4.5,0.0)+'.fits')
	#wav = np.arange(len(sc))*hd['CDELT1'] + hd['CRVAL1']
	teff = float(teff)

	try:
		sc = get_model(teff,logg,feh)
		hd = pyfits.getheader(model_path+get_modname(7000,4.5,0.0)+'.fits')
		mw = np.arange(len(sc))*hd['CDELT1'] + hd['CRVAL1']
	except:
		mw,sc = trilinear_interpolation(teff,logg,feh,model_path)

	for order in range(len(flxs)):
		flxs[order] = clean_strong_lines(wavs[order],flxs[order])

	sc = clean_strong_lines(mw,sc)

	II = np.where(sc != 1)[0]
	JJ = np.where(sc == 1)[0]

	coef = normalize_model(mw[II],sc[II])
	sc  /= np.polyval(coef,mw)
	sc[JJ] = 1.
	mw = ToVacuum(mw)
	weis1 = []
	ccftot = []
	for i in range(wavs.shape[0]):
		#plot(wavs[i],flxs[i])
		scf = flxs[i]
		scw = wavs[i]

		J = np.where(scf!=0)[0]
		scw,scf = scw[J],scf[J]
		I = np.where((mw>scw[0]-100) & (mw<scw[-1]+100))
		tmf = pyasl.fastRotBroad(mw[I], sc[I], 0.5, vsini)
		#plot(mw[I],tmf)
		J = np.where(scf!=1)[0]
		if len(J)>100:
			ccv,ccf = spec_ccf(scw,scf,mw[I],tmf,vmin,vmax,vstep)
			#plot(ccv,ccf)
			#show()
			#ccf = np.array(ccf)
			wei1 = len(np.where(scf!=1)[0])**2
			weis1.append(wei1)
			if len(ccftot)==0:
				ccftot = ccf.copy()*wei1
			else:
				ccftot = np.vstack((ccftot,ccf.copy()*wei1))
	#show()
	weis1 = np.array(weis1)
	ccftot = np.sum(ccftot,axis=0)/ np.sum(weis1)

	p0 = [ccftot.min(),ccv[np.argmin(ccftot)],vsini,ccftot[0]]
	p1, success = scipy.optimize.leastsq(errfunc,p0, args=(ccv,ccftot))

	return p1,ccv,ccftot,fitfunc(p1,ccv)

def calc_bss2(vels,xc,coef, bot_i=0.15, bot_f=0.4, top_i=0.6, top_f=0.9, dt=0.01):
	try:

		I1 = np.where((vels>coef[1]-3*coef[2]) & (vels<coef[1]) )[0]
		I2 = np.where((vels<coef[1]+3*coef[2]) & (vels>coef[1]) )[0]
		I3 = np.where(vels<coef[1]-4*coef[2])[0]
		I4 = np.where(vels>coef[1]+4*coef[2])[0]
		I = np.hstack((I3,I4))
		base = np.median(xc[I])

		xc = base - xc
		xc /= xc.max()


		v1,x1 = vels[I1],xc[I1]
		v2,x2 = vels[I2],xc[I2]
		#plot(v1,x1)
		#plot(v2,x2)
		#show()
		dp = top_f
		vect = []
		while dp >= top_i:
			lb = np.where(x1>dp)[0][0]
			m = (v1[lb] - v1[lb-1])/(x1[lb]-x1[lb-1])
			n = v1[lb] - m*x1[lb]
			bs1 = m*dp+n

			lb = np.where(x2>dp)[0][-1]
			m = (v2[lb] - v2[lb+1])/(x2[lb]-x2[lb+1])
			n = v2[lb] - m*x2[lb]
			bs2 = m*dp+n
			vect.append(0.5*(bs2+bs1))
			dp-=dt
		vect = np.array(vect)

		dp = bot_f
		vecb = []
		while dp >= bot_i:

			lb = np.where(x1>dp)[0][0]
			m = (v1[lb] - v1[lb-1])/(x1[lb]-x1[lb-1])
			n = v1[lb] - m*x1[lb]
			bs1 = m*dp+n

			lb = np.where(x2>dp)[0][-1]
			m = (v2[lb] - v2[lb+1])/(x2[lb]-x2[lb+1])
			n = v2[lb] - m*x2[lb]
			bs2 = m*dp+n
			vecb.append(0.5*(bs2+bs1))
			dp-=dt
		vecb = np.array(vecb)

		return np.median(vecb) - np.median(vect)
	except:
		return -999.0
"""
def lnlike(theta, W, F, Ferr):
	mw,sc = trilinear_interpolation(int(theta[0]),theta[1],theta[2])
	sct  = clean_strong_lines(mw,sc.copy())
	#plot(mw,sc)
	#show()
	coef = normalize_model(mw,sct)
	sc /= np.polyval(coef,mw)

	#print gfd
	mw = ToVacuum(mw)
	mw *= 1 + theta[3]/299792.458

	totD,totM,totE = np.array([]),np.array([]),np.array([])

	for i in range(W.shape[0]):
		scf = F[i]
		scw = W[i]
		scfe = Ferr[i]

		J = np.where(scf!=0)[0]
		scw,scf,scfe = scw[J],scf[J],scfe[J]

		I = np.where((mw>scw[0]-10) & (mw<scw[-1]+10))
		tmf = pyasl.fastRotBroad(mw[I], sc[I], 0.5, theta[4])
		tck = interpolate.splrep(mw[I],tmf,k=1)
		tmf = interpolate.splev(scw,tck)

		tmf = clean_strong_lines(scw,tmf.copy())
		I = np.where(tmf!=1)[0]
		#plot(scw,tmf)
		#plot(scw[I],tmf[I])
		#plot(scw[I],scf[I])

		#show()
		#print gfd

		tmf = tmf[I]
		scf = scf[I]
		scfe = scfe[I]

		tmf /= np.sum(tmf)
		tsf = scf/np.sum(scf)
		tse = scfe*(np.sum(scf)**2)

		totD = np.hstack((totD,tsf))
		totM = np.hstack((totM,tmf))
		totE = np.hstack((totE,tse))
		#plot(scw[I],tsf)
		#plot(scw[I],tmf)
		#plot(scw[I],tsf + 1./np.sqrt(tse))

	#show()
	#print fds
	#print theta
	#show()
	#print gvfd
	#ret = -np.log(2*np.pi) + np.log(np.sum(np.exp(-0.5*((y-model)/yerr)**2)/yerr))
	#ret = -0.5*(np.sum(inv_sigma2*(F-model)**2 - np.log(inv_sigma2)))
	ret = -0.5*(np.sum(totE*(totD-totM)**2 - np.log(totE)))

	#for i in range(len(F)):
	#	errorbar(Y,F[i],yerr=Ferr[i],fmt='b')
	#for j in model:
	#	plot(Y,j,'r')
	#show()
	#print theta, ret
	if np.isnan(ret):
		return -np.inf
	else:
		return ret


def lnprior(theta):
	if 6000 < theta[0] < 9000 and 3.0 < theta[1] < 4.5 and -1 < theta[2] < 0.2 and -500 < theta[3] < 500 and  1. < theta[4] < 500.:
	    return 0.0
	return -np.inf

def lnprob(theta, W,F,Ferr):
	lp = lnprior(theta)
	if not np.isfinite(lp):
		return -np.inf
	return lp + lnlike(theta,W,F,Ferr)
"""
def multiccf(pars):
	teff,logg,feh,vsini=pars[0],pars[1],pars[2],pars[3]
	vmin=-500
	vmax=500.
	vstep=20.
	sc = get_model(teff,logg,feh)
	hd = pyfits.getheader(model_path+get_modname(7000,4.5,0.0)+'.fits')
	wav = np.arange(len(sc))*hd['CDELT1'] + hd['CRVAL1']
	try:
		sc = get_model(teff,logg,feh)
		hd = pyfits.getheader(model_path+get_modname(7000,4.5,0.0)+'.fits')
		mw = np.arange(len(sc))*hd['CDELT1'] + hd['CRVAL1']
	except:
		mw,sc = trilinear_interpolation(teff,logg,feh,model_path)

	sc = clean_strong_lines(mw,sc)

	II = np.where(sc != 1)[0]
	JJ = np.where(sc == 1)[0]

	coef = normalize_model(mw[II],sc[II])
	sc  /= np.polyval(coef,mw)
	sc[JJ] = 1.
	mw = ToVacuum(mw)
	weis1 = []
	ccftot = []
	for i in range(wavs.shape[0]):
		scf = flxs[i].copy()
		scw = wavs[i].copy()

		J = np.where(scf!=0)[0]
		scw,scf = scw[J],scf[J]
		I = np.where((mw>scw[0]-100) & (mw<scw[-1]+100))
		tmf = pyasl.fastRotBroad(mw[I], sc[I], 0.5, vsini)
		#plot(mw[I],tmf)
		J = np.where(scf!=1)[0]
		if len(J)>100:
			ccv,ccf = spec_ccf(scw,scf,mw[I],tmf,vmin,vmax,vstep)
			#ccv,ccf = ccf_fft(scw,scf,mw[I],tmf)
			#plot(ccv,ccf)
			#show()

			wei1 = len(np.where(scf!=1)[0])**2
			weis1.append(wei1)
			if len(ccftot)==0:
				ccftot = ccf.copy()*wei1
			else:
				ccftot = np.vstack((ccftot,ccf.copy()*wei1))
	weis1 = np.array(weis1)
	ccftot = np.sum(ccftot,axis=0)/ np.sum(weis1)


	#print gfds
	#ccftot = np.mean(ccftot,axis=0)
	#print(pars, ccftot.min())
	return ccftot.min()


def get_pars_fr(wavst,flxst,model_patht='../../data/COELHO2014/',npools=4,fixG=1.0):
	for order in range(len(flxst)):
		flxst[order] = clean_strong_lines(wavst[order],flxst[order],mode=1)

	t0 = time.time()

	global wavs,flxs
	global model_path

	wavs,flxs=wavst.copy(),flxst.copy()
	model_path=model_patht


	gt = np.array([6000,7000,8000,9000,10000])
	gg = np.array([2.5,3.0,3.5,4.0,4.5])
	if fixG != -1:
		gg = np.array([fixG])
	gz = np.array([-1,-0.5,0.0,0.2])
	gr = np.array([10.,50.,100.,150.,200.,250.,300.])

	#"""
	tr = np.tile(gr,len(gt)*len(gg)*len(gz))
	tg = np.repeat(np.tile(gg,len(gt)),len(gr)*len(gz))
	tz = np.repeat(np.tile(gz,len(gt)*len(gg)),len(gr))
	tt = np.repeat(gt,len(gg)*len(gr)*len(gz))
	tot = np.vstack((tt,tg,tz,tr)).T

	#for pars in tot:
	#	pars = [8000,4.0,-0.5,40.0]
	#	print(pars, multiccf(pars))

	def _ignore_sigterm():
		import signal; signal.signal(signal.SIGTERM, signal.SIG_IGN)
	p = Pool(npools, initializer=_ignore_sigterm)
	vals = np.array((p.map(multiccf, list(tot))))
	p.terminate()
	I = np.argmin(vals)
	best_vals = tot[I]
	bt,bg,bz,br = best_vals[0],best_vals[1],best_vals[2],best_vals[3]
	#"""
	t1 = time.time()
	print(bt,bg,bz,br, (t1-t0)/60.,'mins')


	#bt,bg,bz,br = 7000.,4.5, 0.2, 100.0
	gt = np.arange(bt-1000,bt+1001,250)
	I = np.where((gt>=6000) & (gt<=10000))[0]
	gt = gt[I]
	gr = np.arange(br-60.,br+61.,20.)
	I = np.where(gr>=10)[0]
	gr = gr[I]

	tr = np.tile(gr,len(gt)*len(gg)*len(gz))
	tg = np.repeat(np.tile(gg,len(gt)),len(gr)*len(gz))
	tz = np.repeat(np.tile(gz,len(gt)*len(gg)),len(gr))
	tt = np.repeat(gt,len(gg)*len(gr)*len(gz))
	tot = np.vstack((tt,tg,tz,tr)).T

	def _ignore_sigterm():
		import signal; signal.signal(signal.SIGTERM, signal.SIG_IGN)
	p = Pool(npools, initializer=_ignore_sigterm)
	vals = np.array((p.map(multiccf, list(tot))))
	p.terminate()
	I = np.argmin(vals)
	best_vals = tot[I]
	bt,bg,bz,br = best_vals[0],best_vals[1],best_vals[2],best_vals[3]
	t2 = time.time()
	print(bt,bg,bz,br, (t2-t1)/60.,'mins')
	#np.savetxt('temp_grid.txt',vals)


	if fixG==-1:
		grid = np.reshape(vals,(len(gt),len(gg),len(gz),len(gr)))
		tckt = interpolate.splrep(gt,np.arange(len(gt)),k=1)
		tckg = interpolate.splrep(gg,np.arange(len(gg)),k=1)
		tckz = interpolate.splrep(gz,np.arange(len(gz)),k=1)
		tckr = interpolate.splrep(gr,np.arange(len(gr)),k=1)

		itckt = interpolate.splrep(np.arange(len(gt)),gt,k=1)
		itckg = interpolate.splrep(np.arange(len(gg)),gg,k=1)
		itckz = interpolate.splrep(np.arange(len(gz)),gz,k=1)
		itckr = interpolate.splrep(np.arange(len(gr)),gr,k=1)

		st = np.arange(gt[0],gt[-1]+1,10.)
		sg = np.arange(gg[0],gg[-1]+0.01,0.1)
		sz = np.arange(gz[0],gz[-1]+0.01,0.1)
		sr = np.arange(gr[0],gr[-1]+1.,5.)

		st = interpolate.splev(st,tckt)
		sg = interpolate.splev(sg,tckg)
		sz = interpolate.splev(sz,tckz)
		sr = interpolate.splev(sr,tckr)

		tr2 = np.tile(sr,len(st)*len(sg)*len(sz))
		tg2 = np.repeat(np.tile(sg,len(st)),len(sr)*len(sz))
		tz2 = np.repeat(np.tile(sz,len(st)*len(sg)),len(sr))
		tt2 = np.repeat(st,len(sg)*len(sr)*len(sz))
		tot2 = np.vstack((tt2,tg2,tz2,tr2))

		zi = ndimage.map_coordinates(grid, tot2, order=3, mode='nearest')
		I  = np.argmin(zi)
		minval = tot2[:,I]

		mint = interpolate.splev(minval[0],itckt)
		ming = interpolate.splev(minval[1],itckg)
		minz = interpolate.splev(minval[2],itckz)
		minr = interpolate.splev(minval[3],itckr)

	else:
		grid = np.reshape(vals,(len(gt),len(gz),len(gr)))
		tckt = interpolate.splrep(gt,np.arange(len(gt)),k=1)
		tckz = interpolate.splrep(gz,np.arange(len(gz)),k=1)
		tckr = interpolate.splrep(gr,np.arange(len(gr)),k=1)

		itckt = interpolate.splrep(np.arange(len(gt)),gt,k=1)
		itckz = interpolate.splrep(np.arange(len(gz)),gz,k=1)
		itckr = interpolate.splrep(np.arange(len(gr)),gr,k=1)

		st = np.arange(gt[0],gt[-1]+1,10.)
		sz = np.arange(gz[0],gz[-1]+0.01,0.1)
		sr = np.arange(gr[0],gr[-1]+1.,5.)

		st = interpolate.splev(st,tckt)
		sz = interpolate.splev(sz,tckz)
		sr = interpolate.splev(sr,tckr)

		tr2 = np.tile(sr,len(st)*len(sz))
		tz2 = np.repeat(np.tile(sz,len(st)),len(sr))
		tt2 = np.repeat(st,len(sr)*len(sz))
		tot2 = np.vstack((tt2,tz2,tr2))

		zi = ndimage.map_coordinates(grid, tot2, order=3, mode='nearest')
		I  = np.argmin(zi)
		minval = tot2[:,I]

		mint = interpolate.splev(minval[0],itckt)
		ming = fixG
		minz = interpolate.splev(minval[1],itckz)
		minr = interpolate.splev(minval[2],itckr)

	#d = {'grid':grid, 'zi':zi, 'tot2':tot2, 'gt':gt, 'gg':gg, 'gz':gz, 'gr':gr}
	#pickle.dump(d,open('temp_dict.pkl','wb'))

	return float(mint),float(ming),float(minz),float(minr)

def plot_CCF_FR(xc_dict,path='XC.pdf'):

	vels       = xc_dict['vels']
	xc_av      = xc_dict['xc_av']
	XCmodelgau = xc_dict['XCmodelgau']
	#refvel     = xc_dict['refvel']
	p1gau      = xc_dict['p1gau']

	f1 = plt.figure()

	pp = PdfPages(path)
	ax1 = f1.add_subplot(111)
	ax1.plot(vels, xc_av,'b.', label='CCF')
	ax1.plot(vels, XCmodelgau,'r-',label='Gaussian fit')

	plt.xlabel('Velocity (km/s)')
	plt.ylabel('XC')
	ax1.axvline(p1gau[1],linestyle=':',color='r')
	ax1.axhline(0.0,linestyle='-')
	plt.title('Average Cross-Correlation Function + Fit')
	handles, labels = ax1.get_legend_handles_labels()
	ax1.legend(handles[::-1], labels[::-1],prop={'size':6})
	pp.savefig()
	pp.close()
	plt.clf()
	pass

"""
def trans_chromosome(chromosome):
	teff = chromosome[0]*100.+chromosome[1]*10.+chromosome[2]
	m = (10000.- 6000.)/999.
	n = 6000.
	teff = teff*m + n
	logg = chromosome[3] + chromosome[4]*0.1
	m = (4.5 - 3.0)/9.9
	n = 3.
	logg = logg*m + n

	feh  = chromosome[5] + chromosome[6]*0.1
	m = (0.2 - -1.)/9.9
	n = -1.
	feh = feh*m + n

	vsini  = chromosome[7]*10. + chromosome[8]
	m = (300. - 10.)/99.
	n = 10.
	vsini = vsini*m + n

	return teff, logg, feh, vsini

global wavs, flxs

def find_pars_GA(wavs,flxs,model_path='../../data/COELHO2014/'):

	def eval_func(chromosome):
		print(list(chromosome))
		teff, logg, feh, vsini = trans_chromosome(chromosome)
		print(teff, logg, feh, vsini)
		pt,vels,ccf,mod = RVforFR(wavs,flxs,teff=teff,logg=logg,feh=feh,vsini=vsini,model_path=model_path)
		score = -ccf.min()
		return score

	genome = G1DList.G1DList(9)
	genome.evaluator.set(eval_func)

	ga = GSimpleGA.GSimpleGA(genome, interactiveMode=True)
	ga.setGenerations(40)
	ga.setMutationRate(0.2)
	ga.setPopulationSize(20)
  	#ga.setCrossoverRate(1.0)
	genome.setParams(rangemin=0, rangemax=9)
	#ga.setMultiProcessing(True)
	ga.evolve(freq_stats=10)
	print(ga.bestIndividual())
	print(trans_chromosome(ga.bestIndividual()))

"""



"""Post-processing API for computing activity indicators on reduced spectra.

Usage from Python:

    from ceres3.postprocess import process_spectrum, process_directory

    # Single file
    result = process_spectrum('/path/to/star_sp.fits', save_1d=True)

    # Whole night
    results = process_directory('/path/to/2019-12-04_red/', save_1d=True, update_fits=True)
"""

import os
import glob
import numpy as np
from astropy.io import fits as pyfits
from ceres3.utils.activity import compute_activity


def process_spectrum(fits_path, instrument=None, save_1d=False,
                     update_fits=False, teff=None):
    """Compute activity indicators for a single reduced spectrum.

    Parameters
    ----------
    fits_path : str
        Path to a CERES *_sp.fits spectrum cube.
    instrument : str, optional
        Override instrument name (default: read from FITS header).
    save_1d : bool
        Save merged 1D spectrum as *_1d.fits alongside the input.
    update_fits : bool
        Write activity values back into the FITS header.
    teff : float, optional
        Override Teff for log R'HK (default: read from FITS header).

    Returns
    -------
    dict with all activity indicator values + metadata.
    """
    hdul = pyfits.open(fits_path)
    spec = hdul[0].data
    header = hdul[0].header

    if instrument is None:
        instrument = header.get('INST', 'feros').lower()

    if teff is None:
        teff = header.get('TEFF', None)
        if teff is not None and teff <= 0:
            teff = None

    out_1d = fits_path.replace('_sp.fits', '_1d.fits') if save_1d else None

    activity = compute_activity(spec, instrument=instrument,
                                output_1d_path=out_1d, teff=teff)

    # Add metadata from header
    activity['filename'] = os.path.basename(fits_path)
    activity['bjd'] = header.get('BJD_OUT', 0.0)
    activity['rv'] = header.get('RV', 0.0)
    activity['rv_err'] = header.get('RV_E', 0.0)

    if update_fits:
        with pyfits.open(fits_path, mode='update') as hdu:
            for key, val in activity.items():
                if key in ('filename', 'spectrum_1d_path'):
                    continue
                try:
                    hdu[0].header[key.upper()] = np.around(val, 6)
                except Exception:
                    pass
            if activity.get('spectrum_1d_path'):
                hdu[0].header['SPEC1D'] = activity['spectrum_1d_path']
            hdu.flush()

    hdul.close()
    return activity


def process_directory(input_dir, instrument=None, save_1d=False,
                      update_fits=False):
    """Compute activity indicators for all reduced spectra in a directory.

    Parameters
    ----------
    input_dir : str
        Path to reduction output directory (looks for proc/*_sp.fits).
    instrument : str, optional
        Override instrument name.
    save_1d : bool
        Save merged 1D spectra.
    update_fits : bool
        Write activity values back into FITS headers.

    Returns
    -------
    list of dicts, one per spectrum.
    """
    input_dir = input_dir.rstrip('/')
    proc_dir = os.path.join(input_dir, 'proc')
    if not os.path.isdir(proc_dir):
        proc_dir = input_dir

    fits_files = sorted(glob.glob(os.path.join(proc_dir, '*_sp.fits')))
    if not fits_files:
        raise FileNotFoundError(f"No *_sp.fits files found in {proc_dir}")

    results = []
    for fpath in fits_files:
        try:
            result = process_spectrum(fpath, instrument=instrument,
                                      save_1d=save_1d, update_fits=update_fits)
            results.append(result)
        except Exception as e:
            results.append({'filename': os.path.basename(fpath), 'error': str(e)})

    return results



"""
Chromospheric activity indicators computed inline from echelle spectra.

Computes S-index (Ca H+K), H-alpha, HeI, and NaI D1/D2 indices directly
from the spec array in memory — no FITS I/O round-trip needed.

Algorithms and constants match Ceres-plusplus (cerespp) exactly.
"""

import numpy as np
from scipy.interpolate import interp1d
from scipy.integrate import trapezoid
from scipy.signal.windows import triang

# ── Line centers (Angstroms) ──────────────────────────────────────────────
CaK = 3933.664
CaH = 3968.47
CaV = 3901.0
CaR = 4001.0
Ha = 6562.808
Hb = 4861.327
HeI = 5875.62
NaID1 = 5895.92
NaID2 = 5889.95
CaIRT1 = 8498.02
CaIRT2 = 8542.09
CaIRT3 = 8662.14
MgIb1 = 5167.321
MgIb2 = 5172.684
MgIb3 = 5183.604

# Instruments that can't compute S-index (wavelength coverage too blue)
S_INDEX_EXCEPTIONS = {'fideos'}
# Instruments that can't compute Ca II IRT (wavelength coverage doesn't reach ~8700 A)
CA_IRT_EXCEPTIONS = {'feros', 'fideos', 'cafe', 'pucheros'}


def _teff_to_bv(teff):
    """Approximate B-V from Teff using Sekiguchi & Fukugita (2000).
    Valid for ~4000-7000 K main sequence stars."""
    # Polynomial inversion of Teff(B-V) relation
    # Teff = 8540 / (B-V + 0.865) for solar metallicity (ballpark)
    return 8540.0 / teff - 0.865


def _noyes_ccf(bv):
    """Noyes et al. (1984) Ccf bolometric correction factor.
    log Ccf = polynomial in (B-V)."""
    x = bv
    if x < 0.63:
        return 1.13 - 10.918 * x + 36.861 * x**2 - 45.648 * x**3 + 18.0 * x**4
    else:
        return -2.117 + 8.204 * x - 8.446 * x**2 + 3.189 * x**3


def _r_phot(bv):
    """Photospheric contribution to R_HK (Noyes et al. 1984).
    log R_phot = polynomial in (B-V)."""
    x = bv
    log_rphot = -4.898 + 1.918 * x**2 - 2.893 * x**3
    return 10**log_rphot


def _get_response(line, width, filt='square'):
    """Build a response filter function centered on a spectral line."""
    if filt == 'square':
        hw = width / 2.0
        w_resp = np.linspace(line - hw, line + hw, 1000)
        resp_f = interp1d(w_resp, np.ones(1000),
                          bounds_error=False, fill_value=0.0)
        return resp_f, hw
    elif filt == 'triangle':
        w_resp = np.linspace(line - width, line + width, 999)
        resp_f = interp1d(w_resp, triang(999, True),
                          bounds_error=False, fill_value=0.0)
        return resp_f, width


def _get_ini_end(wavelength, line, width):
    """Find index range covering line ± width."""
    ini = max(0, np.searchsorted(wavelength, line - width) - 2)
    end = min(len(wavelength), np.searchsorted(wavelength, line + width) + 2)
    return ini, end


def get_line_flux(wavelength, flux, line, width, filt='square', error=None):
    """Compute integrated line flux convolved with a response filter.

    Parameters
    ----------
    wavelength : array — merged 1D wavelength array (Angstroms)
    flux : array — merged 1D flux array
    line : float — line center (Angstroms)
    width : float — bandwidth (full width for square, FWHM for triangle)
    filt : str — 'square' or 'triangle'
    error : array, optional — flux error array

    Returns
    -------
    integrated_flux : float
    sigma : float (only if error is provided)
    """
    resp_f, hw = _get_response(line, width, filt)
    ini, end = _get_ini_end(wavelength, line, hw)

    if end - ini < 4:
        if error is not None:
            return -999.0, -999.0
        return -999.0

    n_pts = max(end - ini, 10)
    w = np.linspace(line - hw, line + hw, n_pts)

    # Interpolate flux onto uniform grid within the line region
    pad = 2
    sl = slice(max(0, ini - pad), min(len(wavelength), end + pad))
    intp = interp1d(wavelength[sl], flux[sl],
                    bounds_error=False, fill_value='extrapolate')
    intp_flux = intp(w)
    response = resp_f(w)

    denom = trapezoid(response, w)
    if denom == 0:
        if error is not None:
            return -999.0, -999.0
        return -999.0

    integrated_flux = trapezoid(intp_flux * response, w) / denom

    if error is not None:
        err_slice = error[ini:end]
        wav_slice = wavelength[ini:end]
        resp_vals = resp_f(wav_slice)
        num = np.sum((err_slice * resp_vals) ** 2)
        den = np.sum(resp_vals ** 2)
        sigma = np.sqrt(num / den) if den > 0 else -999.0
        return integrated_flux, sigma

    return integrated_flux


def merge_echelle(spec, wav_idx=0, flux_idx=5, err_idx=6, snr_idx=8):
    """Merge overlapping echelle orders into a single 1D spectrum.

    Uses S/N crossover between adjacent orders to determine the optimal
    stitching wavelength.

    Parameters
    ----------
    spec : ndarray, shape (n_layers, n_orders, n_pix)
    wav_idx, flux_idx, err_idx, snr_idx : layer indices

    Returns
    -------
    wavelength, flux, error : 1D arrays
    """
    n_orders = spec.shape[1]
    wave_parts, flux_parts, err_parts = [], [], []
    next_start = 0

    for i in range(n_orders - 1, 0, -1):
        wc = spec[wav_idx, i, :]
        wn = spec[wav_idx, i - 1, :]
        snc = spec[snr_idx, i, :]
        snn = spec[snr_idx, i - 1, :]

        # Find S/N crossover point in the overlap region
        overlap_lo = max(wn[0], wc[0])
        overlap_hi = min(wn[-1], wc[-1])

        if overlap_hi <= overlap_lo:
            # No overlap — use the full order
            cur_end = len(wc)
        else:
            wav_grid = np.linspace(overlap_lo, overlap_hi, 1000)
            sn_cur = np.interp(wav_grid, wc, snc)
            sn_nxt = np.interp(wav_grid, wn, snn)
            crossover_wav = wav_grid[np.argmin(np.abs(sn_cur - sn_nxt))]
            cur_end = min(np.searchsorted(wc, crossover_wav), len(wc) - 1)

        wave_parts.append(spec[wav_idx, i, next_start:cur_end])
        flux_parts.append(spec[flux_idx, i, next_start:cur_end])
        err_parts.append(spec[err_idx, i, next_start:cur_end])

        if overlap_hi > overlap_lo:
            next_start = min(np.searchsorted(wn, crossover_wav), len(wn) - 1)
        else:
            next_start = 0

    # Final (bluest) order
    wave_parts.append(spec[wav_idx, 0, next_start:])
    flux_parts.append(spec[flux_idx, 0, next_start:])
    err_parts.append(spec[err_idx, 0, next_start:])

    wavelength = np.concatenate(wave_parts)
    flux = np.concatenate(flux_parts)
    error = np.concatenate(err_parts)

    return wavelength, flux, error


def compute_activity(spec, instrument='feros', output_1d_path=None, teff=None):
    """Compute all activity indicators from an echelle spectrum array.

    Parameters
    ----------
    spec : ndarray, shape (11, n_orders, n_pix)
        The standard CERES output spectrum array.
        Layer 0: wavelength, 5: normalized flux, 6: errors, 8: SNR.
    instrument : str
        Instrument name (used to skip indicators outside wavelength coverage).
    output_1d_path : str, optional
        If provided, save the merged 1D rest-frame spectrum as a FITS file.
    teff : float, optional
        Effective temperature in K. Used to compute log R'HK from S-index.

    Returns
    -------
    dict with keys: s_index, s_index_err, log_rhk, halpha, halpha_err,
                    hbeta, hbeta_err, hei, hei_err, nai_d1d2, nai_d1d2_err,
                    ca_irt, ca_irt_err, mg_ib, mg_ib_err, spectrum_1d_path
    """
    result = {
        's_index': -999.0, 's_index_err': -999.0,
        'log_rhk': -999.0,
        'halpha': -999.0, 'halpha_err': -999.0,
        'hbeta': -999.0, 'hbeta_err': -999.0,
        'hei': -999.0, 'hei_err': -999.0,
        'nai_d1d2': -999.0, 'nai_d1d2_err': -999.0,
        'ca_irt1': -999.0, 'ca_irt1_err': -999.0,
        'ca_irt2': -999.0, 'ca_irt2_err': -999.0,
        'ca_irt3': -999.0, 'ca_irt3_err': -999.0,
        'mg_ib': -999.0, 'mg_ib_err': -999.0,
    }

    # Merge echelle orders into 1D spectrum
    wavelength, flux, error = merge_echelle(spec)

    # Remove bad pixels
    good = (wavelength > 0) & np.isfinite(flux) & np.isfinite(error)
    wavelength, flux, error = wavelength[good], flux[good], error[good]

    if len(wavelength) < 100:
        return result

    # Save merged 1D spectrum if requested
    result['spectrum_1d_path'] = None
    if output_1d_path is not None:
        try:
            from astropy.io import fits as pyfits
            col_wav = pyfits.Column(name='WAVELENGTH', format='D', array=wavelength, unit='Angstrom')
            col_flx = pyfits.Column(name='FLUX', format='D', array=flux)
            col_err = pyfits.Column(name='ERROR', format='D', array=error)
            hdu = pyfits.BinTableHDU.from_columns([col_wav, col_flx, col_err])
            hdu.header['EXTNAME'] = 'SPECTRUM_1D'
            primary = pyfits.PrimaryHDU()
            hdul = pyfits.HDUList([primary, hdu])
            hdul.writeto(output_1d_path, overwrite=True)
            result['spectrum_1d_path'] = output_1d_path
        except Exception:
            pass

    # ── S-index (Ca H+K) ─────────────────────────────────────────────
    if instrument.lower() not in S_INDEX_EXCEPTIONS:
        try:
            NV, sNV = get_line_flux(wavelength, flux, CaV, 20, 'square', error)
            NR, sNR = get_line_flux(wavelength, flux, CaR, 20, 'square', error)
            NK, sNK = get_line_flux(wavelength, flux, CaK, 1.09, 'triangle', error)
            NH, sNH = get_line_flux(wavelength, flux, CaH, 1.09, 'triangle', error)

            if all(v != -999.0 for v in [NV, NR, NK, NH]) and (NR + NV) != 0:
                S = (NH + NK) / (NR + NV)
                sS_num = np.sqrt(sNH**2 + sNK**2)
                sS_den = np.sqrt(sNV**2 + sNR**2)
                sS = S * np.sqrt((sS_num / (NH + NK))**2 + (sS_den / (NR + NV))**2)
                result['s_index'] = float(S)
                result['s_index_err'] = float(sS)
        except Exception:
            pass

    # ── log R'HK (Noyes et al. 1984 calibration from S-index) ────────
    if result['s_index'] != -999.0 and teff is not None and teff > 0:
        try:
            # B-V from Teff using Sekiguchi & Fukugita (2000) approximation
            # Valid for 4000 < Teff < 7000 K
            if 4000 < teff < 7000:
                bv = _teff_to_bv(teff)
                # Noyes et al. 1984: log R'HK from S-index and B-V
                log_ccf = _noyes_ccf(bv)
                r_phot = _r_phot(bv)
                rhk = 1.340e-4 * 10**log_ccf * result['s_index']
                r_prime_hk = rhk - r_phot
                if r_prime_hk > 0:
                    result['log_rhk'] = float(np.log10(r_prime_hk))
        except Exception:
            pass

    # ── H-alpha ───────────────────────────────────────────────────────
    try:
        FHa, sFHa = get_line_flux(wavelength, flux, Ha, 0.678, 'square', error)
        F1, sF1 = get_line_flux(wavelength, flux, 6550.87, 10.75, 'square', error)
        F2, sF2 = get_line_flux(wavelength, flux, 6580.309, 8.75, 'square', error)

        if all(v != -999.0 for v in [FHa, F1, F2]) and (F1 + F2) != 0:
            Halpha = FHa / (0.5 * (F1 + F2))
            sden = np.sqrt(sF1**2 + sF2**2)
            sHalpha = Halpha * np.sqrt((sFHa / FHa)**2 + (sden / (F1 + F2))**2)
            result['halpha'] = float(Halpha)
            result['halpha_err'] = float(sHalpha)
    except Exception:
        pass

    # ── HeI ───────────────────────────────────────────────────────────
    try:
        FHeI, sFHeI = get_line_flux(wavelength, flux, HeI, 0.2, 'square', error)
        F1, sF1 = get_line_flux(wavelength, flux, 5874.5, 0.5, 'square', error)
        F2, sF2 = get_line_flux(wavelength, flux, 5879.0, 0.5, 'square', error)

        if all(v != -999.0 for v in [FHeI, F1, F2]) and (F1 + F2) != 0:
            HelI = FHeI / (0.5 * (F1 + F2))
            sden = np.sqrt(sF1**2 + sF2**2)
            sHelI = HelI * np.sqrt((sFHeI / FHeI)**2 + (sden / (F1 + F2))**2)
            result['hei'] = float(HelI)
            result['hei_err'] = float(sHelI)
    except Exception:
        pass

    # ── NaI D1/D2 ────────────────────────────────────────────────────
    try:
        D1, sD1 = get_line_flux(wavelength, flux, NaID1, 1.0, 'square', error)
        D2, sD2 = get_line_flux(wavelength, flux, NaID2, 1.0, 'square', error)
        L, sL = get_line_flux(wavelength, flux, 5805.0, 10.0, 'square', error)
        R, sR = get_line_flux(wavelength, flux, 6090.0, 20.0, 'square', error)

        if all(v != -999.0 for v in [D1, D2, L, R]) and (L + R) != 0:
            NaI = (D1 + D2) / (L + R)
            sNa_num = np.sqrt(sD1**2 + sD2**2)
            sNa_den = np.sqrt(sL**2 + sR**2)
            sNaI = NaI * np.sqrt((sNa_num / (D1 + D2))**2 + (sNa_den / (L + R))**2)
            result['nai_d1d2'] = float(NaI)
            result['nai_d1d2_err'] = float(sNaI)
    except Exception:
        pass

    # ── H-beta ────────────────────────────────────────────────────────
    try:
        FHb, sFHb = get_line_flux(wavelength, flux, Hb, 0.678, 'square', error)
        F1, sF1 = get_line_flux(wavelength, flux, 4847.0, 10.0, 'square', error)
        F2, sF2 = get_line_flux(wavelength, flux, 4876.0, 10.0, 'square', error)

        if all(v != -999.0 for v in [FHb, F1, F2]) and (F1 + F2) != 0:
            Hbeta = FHb / (0.5 * (F1 + F2))
            sden = np.sqrt(sF1**2 + sF2**2)
            sHbeta = Hbeta * np.sqrt((sFHb / FHb)**2 + (sden / (F1 + F2))**2)
            result['hbeta'] = float(Hbeta)
            result['hbeta_err'] = float(sHbeta)
    except Exception:
        pass

    # ── Ca II IRT (infrared triplet: 8498, 8542, 8662 A) ────────────
    # Individual line indices normalized by continuum on each side
    if instrument.lower() not in CA_IRT_EXCEPTIONS:
        try:
            CL, sCL = get_line_flux(wavelength, flux, 8475.0, 10.0, 'square', error)
            CR, sCR = get_line_flux(wavelength, flux, 8700.0, 10.0, 'square', error)

            if all(v != -999.0 for v in [CL, CR]) and (CL + CR) != 0:
                cont = 0.5 * (CL + CR)
                scont = 0.5 * np.sqrt(sCL**2 + sCR**2)

                for line, key in [(CaIRT1, 'ca_irt1'), (CaIRT2, 'ca_irt2'), (CaIRT3, 'ca_irt3')]:
                    Fl, sFl = get_line_flux(wavelength, flux, line, 1.0, 'square', error)
                    if Fl != -999.0 and cont != 0:
                        idx = Fl / cont
                        sidx = idx * np.sqrt((sFl / Fl)**2 + (scont / cont)**2)
                        result[key] = float(idx)
                        result[key + '_err'] = float(sidx)
        except Exception:
            pass

    # ── Mg I b triplet (5167, 5173, 5184 A) ─────────────────────────
    try:
        Fb1, sFb1 = get_line_flux(wavelength, flux, MgIb1, 0.5, 'square', error)
        Fb2, sFb2 = get_line_flux(wavelength, flux, MgIb2, 0.5, 'square', error)
        Fb3, sFb3 = get_line_flux(wavelength, flux, MgIb3, 0.5, 'square', error)
        CL, sCL = get_line_flux(wavelength, flux, 5140.0, 10.0, 'square', error)
        CR, sCR = get_line_flux(wavelength, flux, 5200.0, 10.0, 'square', error)

        if all(v != -999.0 for v in [Fb1, Fb2, Fb3, CL, CR]) and (CL + CR) != 0:
            MgI = (Fb1 + Fb2 + Fb3) / (1.5 * (CL + CR))
            sMgI_num = np.sqrt(sFb1**2 + sFb2**2 + sFb3**2)
            sMgI_den = np.sqrt(sCL**2 + sCR**2)
            sMgI = MgI * np.sqrt((sMgI_num / (Fb1 + Fb2 + Fb3))**2 + (sMgI_den / (CL + CR))**2)
            result['mg_ib'] = float(MgI)
            result['mg_ib_err'] = float(sMgI)
    except Exception:
        pass

    return result



#!/usr/bin/python
import math
import os
from functools import lru_cache

import numpy
import numpy as np
from astropy.io import fits as pyfits
from multiprocessing import Pool
from scipy import integrate, interpolate, optimize
if not hasattr(integrate, 'simps'):
    integrate.simps = integrate.simpson


@lru_cache(maxsize=128)
def _cached_getdata(path):
    return pyfits.getdata(path)


@lru_cache(maxsize=128)
def _cached_getheader(path):
    return pyfits.getheader(path)


_DATA_DIR = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'utils', 'data') + '/'

import matplotlib.pyplot as plt
from ceres3.utils import vels


def n_Edlen(l):
    """
    Refractive index according to Edlen 1966

    """

    sigma = 1e4 / l
    sigma2 = sigma*sigma
    n = 1 + 1e-8 * (8342.13 + 2406030 / (130-sigma2) + 15997/(38.9-sigma2))

    return n

def n_Morton(l):
    """
    Refractive index used by JJ: Morton 1991, ApJS, 77, 119
    """
    sigma = 1e4 / l
    sigma2 = sigma*sigma
    n = 1 + 6.4328e-5 + 2.94981e-2 / (146.-sigma2) + 2.5540e-4/(41.-sigma2)

    return n

def ToAir(l):
    """

    Transforms input wavelength (A) from Vacuum to Air
    Uses refractive index as per Edlen 1966

    """

    return (l / n_Edlen(l))

def ToVacuum(l):
    """

    Transforms input wavelength (A) from Air to Vacuum
    Uses refractive index as per Edlen 1966


    """

    cond = 1
    l_prev = l.copy()
    while(cond):
        l_new = n_Edlen(l_prev) * l
        if (max(abs(l_new - l_prev)) < 1e-10): cond = 0
        l_prev = l_new

    return l_prev

def el_stl(wam,fl,SLi,SLf):
    for i in range(len(SLi)):
        if SLi[i]>wam[-1]:
            break
        I = numpy.where((wam >= SLi[i]) & (wam<=SLf[i]))[0]
        fl[I]=1.0
    return fl


def _init_corr_p_pool(shared_It, shared_L_Gl, shared_F_Gl, shared_ons, shared_velo2, shared_wat, shared_lux):
    import signal
    signal.signal(signal.SIGTERM, signal.SIG_IGN)

    global It, L_Gl, F_Gl, ons, velo2, wat, lux
    It = shared_It
    L_Gl = shared_L_Gl
    F_Gl = shared_F_Gl
    ons = shared_ons
    velo2 = shared_velo2
    wat = shared_wat
    lux = shared_lux


def corr_p(model_name):
    global It, L_Gl, F_Gl, ons, velo2, wat, lux
    sc = _cached_getdata(model_name)
    FF = sc[It]
    mwa = wat*(1+velo2/lux)

    CCF,NOR = 0,0

    o = 0
    while o < L_Gl.shape[0]:
        #print L[o]
        k = 0
        while k<len(ons):
            if ons[k]==o:
                o=o+1
            k=k+1
        if o >= L_Gl.shape[0]:
            break
        I  = numpy.where(F_Gl[o]!=0.0)[0]
        if I.shape[0] > 0:
            Fl = F_Gl[o][I]
            Ll = L_Gl[o][I]

            I   = numpy.where( (mwa > Ll[0]) & (mwa< Ll[-1]) )[0]
            MML = mwa[I]
            MF  = FF[I]
            #ajustec = continuum.NORM_single(MML, MF, orden=2)
            #plot( MML, MF )
            #plot( MML, numpy.polyval(ajustec, MML))

            #NMF = MF / numpy.polyval(ajustec, MML)
            tckM = interpolate.splrep(MML,MF,k=3,s=0)
            NFM  = interpolate.splev(Ll,tckM,der=0)

            a    = integrate.simps(Fl[1:-1]*NFM[1:-1],Ll[1:-1])
            b    = integrate.simps(Fl[1:-1]*Fl[1:-1],Ll[1:-1])
            c    = integrate.simps(NFM[1:-1]*NFM[1:-1],Ll[1:-1])
            if c < 0:
                plt.plot(MML,MF)
                plt.plot(Ll,Fl)
                plt.plot(Ll,NFM)
                plt.show()
            CCF  = CCF+(a/math.sqrt(b*c))*(Ll[-2]-Ll[1])
            NOR = NOR+Ll[-2]-Ll[1]

        o += 1
    #show()
    return CCF/NOR

def corr(L,F,mwa,FF,ons):
    CCF,NOR = 0,0

    o = 0
    while o < L.shape[0]:
        #print L[o]
        k = 0
        while k<len(ons):
            if ons[k]==o:
                o=o+1
            k=k+1
        if o >= L.shape[0]:
            break

        I  = numpy.where(F[o]!=0.0)[0]
        if I.shape[0] > 0:
            Fl = F[o][I]
            Ll = L[o][I]

            I   = numpy.where( (mwa > Ll[0]) & (mwa< Ll[-1]) )[0]
            MML = mwa[I]
            MF  = FF[I]
            #ajustec = continuum.NORM_single(MML, MF, orden=2)
            #plot( MML, MF )
            #plot( MML, numpy.polyval(ajustec, MML))

            #NMF = MF / numpy.polyval(ajustec, MML)
            tckM = interpolate.splrep(MML,MF,k=3,s=0)
            NFM  = interpolate.splev(Ll,tckM,der=0)

            a    = integrate.simps(Fl[1:-1]*NFM[1:-1],Ll[1:-1])
            b    = integrate.simps(Fl[1:-1]*Fl[1:-1],Ll[1:-1])
            c    = integrate.simps(NFM[1:-1]*NFM[1:-1],Ll[1:-1])
            if c < 0:
                plt.plot(MML,MF)
                plt.plot(Ll,Fl)
                plt.plot(Ll,NFM)
                plt.show()
            CCF  = CCF+(a/math.sqrt(b*c))*(Ll[-2]-Ll[1])
            NOR = NOR+Ll[-2]-Ll[1]

        o += 1
    #show()
    return CCF/NOR

def bad_orders(L,bl):
    bo = []

    o = 0
    while o < L.shape[0]:
        i=0
        while i<len(bl):
            I = numpy.where((L[o]>bl[i][0]) & (L[o]<bl[i][1]))[0]
            if I.shape[0]!=0:
                bo.append(o)
            i = i+1
        o=o+1
    return bo

def get_name(t,g,z):
    T = str(int(t))

    if g == 0.0:
        G = '00'
    elif g == 0.5:
        G = '05'
    else:
        G = str(int(g*10))

    if z < 0:
        siZ = 'm'
        modz = -z
    else:
        siZ = 'p'
        modz = z

    if modz == 0.0:
        Z = '00'
    elif modz == 0.2:
        Z = '02'
    elif modz == 0.5:
        Z = '05'
    else:
        Z = str(int(modz*10))

    return f'{T}_{G}_{siZ}{Z}p00.ms.fits'

def orden(V,W):
    j=0
    lar = len(V)

    while j<lar-1:
        p = V[j]
        pp = W[j]
        i = j+1
        lu = j
        while i < lar:
            if V[i]<p:
                p = V[i]
                pp = W[i]
                lu = i
            i=i+1
        V[lu]=V[j]
        W[lu]=W[j]
        V[j]=p
        W[j]=pp
        j=j+1
    return V,W

def gauss1(params,x):
    C = params[0]
    A = params[1]
    med = params[2]
    sig = params[3]
    g = C+A*numpy.exp(-0.5*(x-med)*(x-med)/(sig*sig))
    return g

def res_gauss1(params,g,x):
    return g-gauss1(params,x)

def CCF(spec, model_path='/dummy/path/',doplot = False, plot_dir = '/home/rabrahm/',plot_name = 'MY_LUP',Li=4500.,Lf=6300.,npools=1,base=_DATA_DIR):
    """
    This function finds an aproximation to the stellar parameters (Teff, log(g), [Fe/H])
    of the input echelle spectrum using a CCF with model spectra. This code also is
    constructed to find the radial velocity of the star and v*sin(i).
    """
    lux = 299792.458

    L1,F1 = spec[0,:,:], spec[5,:,:]
    for i in range(F1.shape[0]):
        I = np.where(np.isnan(F1[i])==True)[0]
        F1[i][I] = 1.

    #width_path = '/home/rabrahm/Desktop/corr2/'
    #slines_path = '/home/rabrahm/Desktop/corr2/'

    SLi,SLf = numpy.loadtxt(base+'lines2.dat',dtype=None,unpack=True)

    SLi = ToVacuum(SLi)
    SLf = ToVacuum(SLf)

    AT,AG,AZ,A000,A025,A050,A075,A100,A150,A200,A250,A300,A350,A400,A450,A500 =\
            numpy.loadtxt(base+'anchos50000.dat',dtype=None,unpack=True)

    vsini = [0.0,2.5,5.0,7.5,10.0,15.0,20.0,25.0,30.0,35.0,40.0,45.0,50.0]

    or01 = 0
    for i in range(L1.shape[0]):
        I = numpy.where(L1[i] < Lf)[0]
        if len(I) > 0:
            or01 = i
            break

    or02 = L1.shape[0]-1
    for i in range(L1.shape[0]):
        I = numpy.where(L1[i] < Li)[0]
        if len(I) > 0:
            or02 = i
            break

    or03 = 0
    for i in range(L1.shape[0]):
        I = numpy.where(L1[i] < 6250.0)[0]
        if len(I) > 0:
            or03 = i
            break

    or04 = L1.shape[0]-1
    for i in range(L1.shape[0]):
        I = numpy.where(L1[i] < 5500.0)[0]
        if len(I) > 0:
            or04 = i
            break

    or05 = 0
    for i in range(L1.shape[0]):
        I = numpy.where(L1[i] < 5190.0)[0]
        W = numpy.where(L1[i] < 5170.0)[0]
        if len(I) > 0 and len(W) > 0:
            or05 = i
            break
    #print(or01,or02,or03,or04,or05)
    guess = [1.0,1.0,1.0]

    L = L1[or01:or02]
    F = F1[or01:or02]
    Lm = L1[or03:or04]
    Fm = F1[or03:or04]
    Lg = L1[or05]
    Fg = F1[or05]


    bad_lines = [[6860,6900],[6550,6580],[6270,6320],[4850,4880]]
    ons = bad_orders(L,bad_lines)

    modi = 'R_0.0_5000_30_p00p00.ms.fits'
    #print('Radial velocity calculation via CCF with: '+modi)

    sci = _cached_getdata(model_path+'vsini_0.0/'+modi)
    hdi = _cached_getheader(model_path+'vsini_0.0/'+modi)

    wam1 = ToVacuum(numpy.arange(len(sci))*hdi['CD1_1']+hdi['CRVAL1'])

    Im = numpy.where((wam1 > 5400.0) & (wam1 < 6350.0))[0]
    wam = wam1[Im]
    flm = sci[Im]

    Ig = numpy.where((wam1 > 5100.0) & (wam1 < 5250.0))[0]
    wag = wam1[Ig]

    It = numpy.where((wam1 > 4000.0) & (wam1 < 7500.0))[0]
    wat = wam1[It]


    for i in range(Lm.shape[0]):
        I = numpy.where((Fm[i] != 0.0) & (Fm[i] < 2.0))[0]
        Ls = Lm[i][I]
        Fs = Fm[i][I]

        I = numpy.where((wam > Ls[0]-5.0) & (wam < Ls[-1]+5.0))[0]
        MLs = wam[I]
        MFs = flm[I]

        vv,cc = vels.CCF(MLs,MFs,Ls,Fs,-200.0,200.0)

        if i == 0:
            ccf = numpy.array(cc)*(Ls[-1]-Ls[0])
            nor = Ls[-1]-Ls[0]
        else:
            ccf = ccf + numpy.array(cc)*(Ls[-1]-Ls[0])
            nor = nor + (Ls[-1]-Ls[0])

    ccf = ccf/nor
    vv = numpy.array(vv)

    B = 0.5*(ccf[0]+ccf[-1])
    A = numpy.max(ccf)-B
    med = vv[numpy.argmax(ccf)]
    sig = 20.0
    guess1 = numpy.array([B,A,med,sig],dtype='float64')
    ajustep=optimize.leastsq(res_gauss1,guess1,args=(ccf,vv))
    velo = ajustep[0][2]

    #print('The initial radial velocity is: '+str(velo)+' km/s')

    #print('Determining parameters of the initial model')

    vecti = [3500,4000,4500,5000,5500,6500]
    vecgi = [0.0,1.5, 3.0,4.5]
    #veczi = [-2.0,-1.0,0.0]
    veczi = [-1.0,0.0]
    ccmax = 0
    names = []
    cc = 0
    nor = 0
    for g in vecgi:
        for z in veczi:
            for t in vecti:
                nam = get_name(t,g,z)
                names.append(nam)
    namemax = get_name(5500,4.5,0.0)
    TEI = 5500
    MEI = 0.0
    LGI = 4.5
    for nam in names:
        if ( (os.access(model_path+'vsini_0.0/R_0.0_'+nam,os.F_OK) == True)):
            mod = _cached_getdata(model_path+'vsini_0.0/R_0.0_'+nam)
            flm = mod[Im]
            cc = 0.0
            nor = 0.0
            for o in range(Lm.shape[0]):
                I = numpy.where(Fm[i] != 0.0)[0]
                Ls = Lm[i][I]
                Fs = Fm[i][I]
                I = numpy.where((wam > Ls[0]-5.0) & (wam < Ls[-1]+5.0))[0]
                MLs = wam[I]*(1+velo/lux)
                MFs = flm[I]

                tck = interpolate.splrep(MLs,MFs,k=3,s=0)
                NMFs = interpolate.splev(Ls,tck,der=0)

                cc = cc + integrate.simps(NMFs[1:-1]*Fs[1:-1],Ls[1:-1])/math.sqrt(integrate.simps(Fs[1:-1]*Fs[1:-1],Ls[1:-1])*integrate.simps(NMFs[1:-1]*NMFs[1:-1],Ls[1:-1]))*(Ls[-1]-Ls[0])
                nor = nor + (Ls[-1]-Ls[0])
            cc = cc/nor

            if cc >=ccmax:
                ccmax = cc
                namemax = nam

    mod = _cached_getheader(model_path+'vsini_0.0/R_0.0_'+namemax)
    TEI = mod['TEFF']

    #print('Teff (initial) = '+str(TEI)+' K')



    if TEI <= 4000:
        rot = 5.0
        LGI = 3.0
        MTI = 0.0
        late = True
        velo2 = velo

    else:
        late = False
        t = TEI
        vecgi  = [1.0,2.0,3.0,4.0]
        #veczi  = [-2.0,-1.0,0.0]
        veczi = [-1.0,0.0]
        dif = 1000.0

        for z in veczi:
            vals = []
            di = 1000
            meg = 3.0
            for g in vecgi:
                nam = get_name(t,g,z)
                if ( (os.access(model_path+'vsini_0.0/R_0.0_'+nam,os.F_OK) == True)):
                    mod = _cached_getdata(model_path+'vsini_0.0/R_0.0_'+nam)
                    flm = mod[Im]
                    flm = el_stl(wam,flm,SLi,SLf)
                    intfl = 0.0
                    intflm = 0.0
                    di = 1000
                    for o in range(Lm.shape[0]):
                        I = numpy.where(Fm[i] != 0.0)[0]
                        Ls = Lm[i][I]
                        Fs = Fm[i][I]
                        Fs = el_stl(Ls,Fs,SLi*(1+velo/lux),SLf*(1+velo/lux))
                        I = numpy.where((wam > Ls[0]-5.0) & (wam < Ls[-1]+5.0))[0]
                        MLs = wam[I]*(1+velo/lux)
                        MFs = flm[I]
                        tck = interpolate.splrep(MLs,MFs,k=3,s=0)
                        NMFs = interpolate.splev(Ls,tck,der=0)

                        intfl = intfl + integrate.simps(Fs[1:-1]*Fs[1:-1],Ls[1:-1])
                        intflm = intflm + integrate.simps(NMFs[1:-1]*NMFs[1:-1],Ls[1:-1])
                    intfl = math.sqrt(intfl)
                    intflm = math.sqrt(intflm)
                    dif1 = math.sqrt((intflm-intfl)**2)
                    vals.append(dif1)
                    if dif1 < di:
                        di = dif1
                        meg = g
            dif2 = numpy.mean(vals)
            #print(z,dif2)
            if dif2<dif:
                dif =dif2
                LGI = meg
                MEI = z
                TEI = t


        #print('[Fe/H] (initial) = '+str(MEI))

        vecgi  = [0.5,1.5,2.5,3.5,4.5]
        cfi2 = 0.0
        for g in vecgi:
            nam = get_name(TEI,g,MEI)
            if ( (os.access(model_path+'vsini_0.0/R_0.0_'+nam,os.F_OK) == True)):
                mod = _cached_getdata(model_path+'vsini_0.0/R_0.0_'+nam)

                mflg = mod[Ig]

                I = numpy.where(Fg != 0.0)[0]
                Ls = Lg[I]
                Fs = Fg[I]

                I = numpy.where((wag > Ls[0]-5.0) & (wag < Ls[-1]+5.0))[0]
                MLs = wag[I]*(1+velo/lux)
                MFs = mflg[I]
                tck = interpolate.splrep(MLs,MFs,k=3,s=0)
                NMFs = interpolate.splev(Ls,tck,der=0)

                cc2 = integrate.simps(Fs[1:-1]*NMFs[1:-1], Ls[1:-1])/math.sqrt(integrate.simps(Fs[1:-1]*Fs[1:-1], Ls[1:-1])*integrate.simps(NMFs[1:-1]*NMFs[1:-1], Ls[1:-1]))
                #print(g,cc2)
                if cc2 > cfi2:
                    cfi2 = cc2
                    LGI  = g

        #print('Log(g) (initial) = '+str(LGI))

    itera = 0
    maximo = 0
    calculated = []
    rotss = []
    vecR = []
    vecT  = []
    vecG  = []

    vecZ  = []
    vecCF = []
    while itera < 4:

        if late==False:

            MOG = get_name(TEI,LGI,MEI)
            sc = _cached_getdata(model_path+'vsini_0.0/R_0.0_'+MOG)

            #print('-Calculating radial shift and v*sin(i) with model: '+MOG)

            flm = sc[Im]
            ies = []
            flm = el_stl(wam,flm,SLi,SLf)
            for i in range(Lm.shape[0]):
                I = numpy.where(Fm[i] != 0.0)[0]
                Ls = Lm[i][I]
                Fs = Fm[i][I]
                Fs = el_stl(Ls,Fs,SLi*(1+velo/lux),SLf*(1+velo/lux))
                I = numpy.where((wam > Ls[0]-5.0) & (wam < Ls[-1]+5.0))[0]
                ies.append(I)
                MLs = wam[I]
                MFs = flm[I]
                vv,cc = vels.CCF(MLs,MFs,Ls,Fs,-200,200)
                if i == 0:
                    ccf = numpy.array(cc)*(Ls[-1]-Ls[0])
                    nor = Ls[-1]-Ls[0]
                else:
                    ccf = ccf + numpy.array(cc)*(Ls[-1]-Ls[0])
                    nor = nor + (Ls[-1]-Ls[0])
            nor2=1.
            ccf = ccf/nor
            vv = numpy.array(vv)
            B = 0.5*(ccf[0]+ccf[-1])
            A = numpy.max(ccf)-B
            med = vv[numpy.argmax(ccf)]
            sig = 20.0
            guess1 = np.array([B,A,med,sig],dtype='float64')
            ajustep=optimize.leastsq(res_gauss1,guess1,args=(ccf,vv))
            velo2 = ajustep[0][2]
            sig2 = ajustep[0][3]
            sig2 = math.sqrt(sig2*sig2)
            #plt.plot(vv,ccf,vv,gauss1(ajustep[0],vv))
            #plt.show()
            #print(f'radial velocity = {velo2} km/s')
            #print(f'Sigma = {sig2} km/s')
            """

            vi = 0.0
            difsigmin = 1000
            while vi <= 20.0:
                ai = 0
                if ( (os.access(model_path+'vsini_'+str(vi)+'/R_'+str(vi)+'_'+MOG,os.F_OK) == True)):
                    modt = _cached_getdata(model_path+'vsini_'+str(vi)+'/R_'+str(vi)+'_'+MOG)
                    flm2 = modt[Im]
                    flm2 = el_stl(wam,flm2,SLi,SLf)
                    for I in ies:
                        Fs = flm[I]
                        Ls = wam[I]
                        Fs = el_stl(Ls,Fs,SLi,SLf)
                        MFs = flm2[I]
                        vv2,cc2 = vels.CCF(Ls,MFs,Ls,Fs,-200,200)
                        if ai == 0:
                            ccf2 = numpy.array(cc2)*(Ls[-1]-Ls[0])
                            nor2 = Ls[-1]-Ls[0]
                        else:
                            ccf2 = ccf2 + numpy.array(cc2)*(Ls[-1]-Ls[0])
                            nor2 = nor2 + (Ls[-1]-Ls[0])
                        ai += 1
                    cc2 = ccf2/nor2
                    vv2 = numpy.array(vv2)
                    B3 = 0.5*(cc2[0]+cc2[-1])
                    A3 = numpy.max(cc2)-B3
                    med3 = 0.0
                    sig3 = 20.0
                    guess1 = [B3,A3,med3,sig3]
                    ajustep=optimize.leastsq(res_gauss1,guess1,args=(cc2,numpy.array(vv2)))
                    cte3 = ajustep[0][0]
                    no3 = ajustep[0][1]
                    med3 = ajustep[0][2]
                    sig3 = ajustep[0][3]

                    #plt.plot(vv2,cc2)

                    print(vi,sig3)
                    difsig = math.sqrt((sig2-sig3)**2)
                    if difsig < difsigmin:
                        difsigmin = difsig
                        rot = vi
                ai +=1

                if vi <= 7.5:
                    vi = vi+2.5
                elif vi < 50.0:
                    vi = vi+5.0
                else:
                    break

            #plt.show()
            """
            if sig2 >= vsini[0]:
                I = numpy.where((AT == TEI) & (AG == LGI) & (AZ == MEI))[0][0]
                anchos = numpy.array([A000[I],A025[I],A050[I],A075[I],A100[I],A150[I],A200[I],A250[I],A300[I],A350[I],A400[I],A450[I],A500[I]])
                kis = 0
                while kis < len(anchos)-1:
                    if anchos[kis]>anchos[kis+1]:
                        portemp = anchos[kis]
                        anchos[kis] = anchos[kis+1]
                        anchos[kis+1] = portemp
                    kis+=1
                tck = interpolate.splrep(anchos,numpy.array(vsini),k=3,s=0)
                calrot = interpolate.splev(sig2,tck,der=0)
                difs = (numpy.array(vsini) - calrot)**2
                AI = int(numpy.argmin(difs))
                rot = vsini[AI]
            else:
                rot = vsini[0]
                calrot = 0.0

            #rot = 2.5
            #print(f'v*sin(i) = {calrot} km/s')

        RI = numpy.where(numpy.array(rotss) == rot)[0]

        if len(RI) > 0:
            break

        else:

            model_path1 = model_path+"vsini_"+str(rot)+"/"

            """ conociendo la velocidad radial realizo una busqueda gruesa de parametros estelares calculando el maximo de la CCF"""
            #print("-Searching the optimal stellar model")

            #maxT,maxG,maxZ,maxCor,maxvs = 5500,4.0,0.0,0.0,0.0
            maxCor = 0.0

            if TEI >= 3500 and TEI <=4250:
                modT = [3500,4000,4500,5000]
            else:
                modT = [4000,4500,5000,5500,6000,6500,7000]

            modG = [1.0,2.5,4.0]
            #modZ = [-2.0,-1.0,0.0]
            modZ = [-1.0,0.0]
            MOD,MOD2 = [],[]

            i=0
            while i < len(modZ):
                ig = 0
                while ig < len(modG):
                    it = 0
                    while it < len(modT):
                        MK = f'R_{rot}_{get_name(modT[it],modG[ig],modZ[i])}'
                        if os.access(model_path1+MK,os.F_OK):
                            MOD.append(MK)
                            MOD2.append(model_path1+MK)
                        it = it+1
                    ig =ig+1
                i = i+1

            MOD = np.array(MOD)
            MOD2 = np.array(MOD2)

            all_pars = np.vstack((MOD,np.zeros(len(MOD))+velo2))
            L_Gl,F_Gl =L.copy(),F.copy()
            with Pool(npools, initializer=_init_corr_p_pool, initargs=(It, L_Gl, F_Gl, ons, velo2, wat, lux)) as p:
                xc_values = np.array(p.map(corr_p, MOD2))
            I_min = np.argmax(xc_values)
            hd = _cached_getheader(MOD2[I_min])
            maxCor = xc_values[I_min]
            maxG   = hd['LOG_G']
            maxT   = hd['TEFF']
            maxZ   = hd['FEH']
            maxvs  = hd['VSINI']
            #print(MOD2[I_min])
            """
            print(xc_values)
            print(MOD2[I_min],I_min)
            print(dfgh)

            for m in MOD:

                hd = _cached_getheader(model_path1+m)
                T  = hd['TEFF']
                G  = hd['LOG_G']
                Z  = hd['FEH']
                vs = hd['VSINI']

                sc = _cached_getdata(model_path1+m)
                FF = sc[It]
                mwa = wat*(1+velo2/lux)
                NCCF = corr(L,F,mwa,FF,ons)

                #print(T,G,Z,vs,NCCF)

                if NCCF > maxCor:
                    maxCor = NCCF
                    maxG   = G
                    maxT   = T
                    maxZ   = Z
                    maxvs  = vs
                elif NCCF<0:
                    print("Problem with spectrum!!! -> Negative value of CCF.")
                    maxG   = 4.5
                    maxT   = 5500
                    maxZ   = 0
                    maxvs  = 0
            """



            #print('maxgrueso',maxT,maxG,maxZ,maxvs)


            """ A partir de los parametros encontrados en la busqueda gruesa procedo a constrenir los limites de la busqueda fina"""


            #Tfin,Gfin,Zfin,rotfin,velfin = 5500,4.0,0.0,0.0,0.0
            """ Ahora se buscan los par'ametros estelares optimos mediante una exploracion fina"""

            if maxT == 3500 or maxT == 4000:
                modT = [3500,3750,4000,4250,4500]

            elif maxT == 7000 or maxT == 6500:
                modT = [6000,6250,6500,6750,7000]
            else:
                modT = [maxT-750,maxT-500,maxT-250,maxT,maxT+250,maxT+500,maxT+750]

            if maxG == 1.0:
                modG = [0.0,0.5,1.0,1.5,2.0,2.5]
            if maxG == 2.5:
                modG = [1.0,1.5,2.0,2.5,3.0,3.5,4.0]
            if maxG == 4.0:
                modG = [2.5,3.0,3.5,4.0,4.5,5.0]

            if maxZ == -2.0:
                modZ = [-2.5,-2.0,-1.5,-1.0]
            if maxZ == -1.0:
                modZ = [-1.5,-1.0,-0.5,0.0]
            if maxZ == 0.0:
                modZ = [-0.5,0.0,0.2,0.5]

            MOD,MOD2 = [],[]
            for i in modZ:
                for ig in modG:
                    for it in modT:
                        MK = f'R_{rot}_{get_name(it,ig,i)}'
                        if os.access(model_path1+MK,os.F_OK):
                            MOD.append(MK)
                            MOD2.append(model_path1+MK)
            MOD2 = np.array(MOD2)

            L_Gl,F_Gl =L.copy(),F.copy()
            with Pool(npools, initializer=_init_corr_p_pool, initargs=(It, L_Gl, F_Gl, ons, velo2, wat, lux)) as p:
                xc_values = np.array(p.map(corr_p, MOD2))
            I_min = np.argmax(xc_values)
            hd = _cached_getheader(MOD2[I_min])
            maxCor = xc_values[I_min]
            maxG   = hd['LOG_G']
            maxT   = hd['TEFF']
            maxZ   = hd['FEH']
            maxvs  = hd['VSINI']
            #print(MOD2[I_min])
            """
            #maxT,maxG,maxZ,maxCor = 5500,4.0,0,0
            maxCor = 0.0
            for m in MOD:
                calculated.append(m)
                hd = _cached_getheader(model_path1+m)
                T  = hd['TEFF']
                G  = hd['LOG_G']
                Z  = hd['FEH']

                sc = _cached_getdata(model_path1+m)
                FF = sc[It]
                mwa = wat*(1+velo2/lux)

                NCCF = corr(L,F,mwa,FF,ons)

                vecT.append(T)
                vecG.append(G)
                vecZ.append(Z)
                vecR.append(rot)
                vecCF.append(NCCF)

                #print(T,G,Z,NCCF)

                if NCCF > maxCor:
                    maxCor = NCCF
                    maxT   = T
                    maxG   = G
                    maxZ   = Z

                elif NCCF < 0:
                    print("Problem with spectrum!!! -> Negative value of CCF.")
                    maxG   = 4.5
                    maxT   = 5500
                    maxZ   = 0
                    maxvs  = 0
            """
            #print("maxfino",maxT,maxG,maxZ,maxCor)

            TEI = maxT
            LGI = maxG
            MEI = maxZ

            ultrot = rot

            if maxCor > maximo:
                maximo = maxCor
                Tfin,Gfin,Zfin,rotfin,velfin = maxT,maxG,maxZ,rot,velo2
                #Tf,Gf,Zf = intert,interg,maZ

            late = False
            rotss.append(rot)


        itera = itera+1
    if maximo == 0:
        Tfin,Gfin,Zfin,rotfin,velfin = 5500, 4.5,0,0,velo2
    #print(f'Pars fase 0: {Tfin} {Gfin} {Zfin} {rotfin} {maximo}')
    mejor = False
    if rotfin == 0.0:
        nrot = [0.0,2.5]
    elif rotfin == 2.5:
        nrot = [0.0,2.5,5.0]
    elif rotfin == 5.0:
        nrot = [2.5,5.0,7.5]
    elif rotfin == 7.5:
        nrot = [5.0,7.5,10.0]
    else:
        nrot = [rotfin-5.0,rotfin,rotfin+5.0]

    if Tfin == 3500:
        nT = [3500,3750,4000]
    elif Tfin == 7000:
        nT = [6500,6750,7000]
    else:
        nT = [Tfin-250,Tfin,Tfin+250]

    if Gfin == 0.0:
        nG = [0.0,0.5,1.0]
    elif Gfin == 0.5:
        nG = [0.0,0.5,1.0]
    elif Gfin == 4.5:
        nG = [4.0,4.5,5.0]
    elif Gfin == 5.0:
        nG = [4.0,4.5,5.0]
    else:
        nG = [Gfin-0.5,Gfin,Gfin+0.5]

    if Zfin == -2.5:
        nZ = [-2.5,-2.0,-1.5]
    elif Zfin == 0.5:
        nZ = [0.0,0.2,0.5]
    elif Zfin == 0.2:
        nZ = [0.0,0.2,0.5]
    elif Zfin == 0.0:
        nZ = [-0.5,0.0,0.2,0.5]
    else:
        nZ = [Zfin-0.5,Zfin,Zfin+0.5]

    for v in nrot:
        model_path2 = model_path+'vsini_'+str(v)+'/'
        names = []
        calc = numpy.array(calculated)

        for t in nT:
            for g in nG:
                for z in nZ:
                    nam = f'R_{v}_{get_name(t,g,z)}'
                    I = numpy.where(calc == nam)[0]
                    if len(I)==0 and os.access(model_path2+nam,os.F_OK):
                        names.append(nam)

        for fits in names:
            calculated.append(fits)
            hd = _cached_getheader(model_path2+fits)
            T  = hd['TEFF']
            G  = hd['LOG_G']
            Z  = hd['FEH']

            sc = _cached_getdata(model_path2+fits)
            FF = sc[It]
            mwa = wat*(1+velfin/lux)

            NCCF = corr(L,F,mwa,FF,ons)

            vecT.append(T)
            vecG.append(G)
            vecZ.append(Z)
            vecCF.append(NCCF)
            vecR.append(v)
            #print(T,G,Z,NCCF)

            if NCCF > maximo:
                mejor = True
                maximo = NCCF
                Tfin   = T
                Gfin   = G
                Zfin   = Z
                rotfin = v

        #print(Tfin,Gfin,Zfin,rotfin,maximo)
        maximCCF = maximo

    total_inter = 1
    if total_inter == 0:
        deltaV = 0.5
        deltaG = 0.05
        deltaT = 50.0
        deltaZ = 0.1

        vCF = numpy.array(vecCF)
        vvT = numpy.array(vecT)
        vvG = numpy.array(vecG)
        vvZ = numpy.array(vecZ)
        vvV = numpy.array(vecR)

        ejeT = numpy.arange(numpy.min(nT),numpy.max(nT)+deltaT,deltaT)
        ejeV = numpy.arange(numpy.min(nrot),numpy.max(nrot)+deltaV,deltaV)
        ejeG = numpy.arange(numpy.min(nG),numpy.max(nG)+deltaG,deltaG)
        ejeZ = numpy.arange(numpy.min(nZ),numpy.max(nZ)+deltaZ,deltaZ)

        lejeV = len(ejeV)
        lejeG = len(ejeG)
        lejeT = len(ejeT)
        lejeZ = len(ejeZ)

        matCCF = numpy.zeros([lejeT,lejeG,lejeZ,lejeV],float)

        for v in nrot:
            posV = int(round((v-ejeV[0])/deltaV))
            for z in nZ:
                posZ = int(round((z-ejeZ[0])/deltaZ))
                for g in nG:
                    posG = int(round((g-ejeG[0])/deltaG))
                    I = numpy.where((vvV == v) & (vvG ==g) & (vvZ == z))[0]
                    if len(I) > 0:
                        vTt,vCFt = orden(vvT[I],vCF[I])

                        if len(I)>3:
                            tck  = interpolate.splrep(vTt,vCFt,k=3,s=0)
                            ynew = interpolate.splev(ejeT,tck,der=0)
                        elif len(I) > 1:
                            tck  = interpolate.splrep(vTt,vCFt,k=len(I)-1,s=0)
                            ynew = interpolate.splev(ejeT,tck,der=0)
                        else:
                            ynew = numpy.zeros(len(ejeT),float)+vCFt[0]

                        matCCF[:,posG,posZ,posV]=ynew

        for v in nrot:
            posV = int(round((v-ejeV[0])/deltaV))
            for z in nZ:
                posZ = int(round((z-ejeZ[0])/deltaZ))
                for t in ejeT:
                    posT = int(round((t-ejeT[0])/deltaT))
                    y1 = matCCF[posT,:,posZ,posV]
                    I = numpy.where(y1 != 0.0)[0]
                    y1b = y1[I]
                    x1b = ejeG[I]
                    if len(I) > 0:
                        if len(I)>3:
                            tck  = interpolate.splrep(x1b,y1b,k=3,s=0)
                            ynew = interpolate.splev(ejeG,tck,der=0)
                        elif len(I) > 1:
                            tck  = interpolate.splrep(x1b,y1b,k=len(I)-1,s=0)
                            ynew = interpolate.splev(ejeG,tck,der=0)
                        else:
                            ynew = numpy.zeros(len(ejeG),float)+y1b[0]
                        matCCF[posT,:,posZ,posV]=ynew
        for v in nrot:
            posV = int(round((v-ejeV[0])/deltaV))
            for t in ejeT:
                posT = int(round((t-ejeT[0])/deltaT))
                for g in ejeG:
                    posG = int(round((g-ejeG[0])/deltaG))
                    y1 = matCCF[posT,posG,:,posV]
                    I = numpy.where(y1 != 0.0)[0]
                    y1b = y1[I]
                    x1b = ejeZ[I]
                    if len(I) > 0:
                        if len(I)>3:
                            tck  = interpolate.splrep(x1b,y1b,k=3,s=0)
                            ynew = interpolate.splev(ejeZ,tck,der=0)
                        elif len(I) > 1:
                            tck  = interpolate.splrep(x1b,y1b,k=len(I)-1,s=0)
                            ynew = interpolate.splev(ejeZ,tck,der=0)
                        else:
                            ynew = numpy.zeros(len(ejeZ),float)+y1b[0]
                        matCCF[posT,posG,:,posV]=ynew

        for t in ejeT:
            posT = int(round((t-ejeT[0])/deltaT))
            for g in ejeG:
                posG = int(round((g-ejeG[0])/deltaG))
                for z in ejeZ:
                    posZ = int(round((z-ejeZ[0])/deltaZ))
                    y1 = matCCF[posT,posG,posZ,:]
                    I = numpy.where(y1 != 0.0)[0]
                    y1b = y1[I]
                    x1b = ejeV[I]
                    if len(I) > 0:
                        if len(I)>3:
                            tck  = interpolate.splrep(x1b,y1b,k=3,s=0)
                            ynew = interpolate.splev(ejeV,tck,der=0)
                        elif len(I) > 1:
                            tck  = interpolate.splrep(x1b,y1b,k=len(I)-1,s=0)
                            ynew = interpolate.splev(ejeV,tck,der=0)
                        else:
                            ynew = numpy.zeros(len(ejeV),float)+y1b[0]
                        matCCF[posT,posG,posZ,:]=ynew

        I = numpy.where(matCCF == numpy.max(matCCF))

        intert = ejeT[I[0]]
        interg = ejeG[I[1]]
        interz = ejeZ[I[2]]
        interv = ejeV[I[3]]
        maximCCF = numpy.max(CCF)
        #print(f'interp {intert} {interg} {interz} {interv}')

    elif total_inter == 1:
        if Tfin == 3500:
            nT = [3500,3750,4000,4250]
        elif Tfin == 3750:
            nT = [3500,3750,4000,4250]
        elif Tfin == 7000:
            nT = [6250,6500,6750,7000]
        elif Tfin == 6750:
            nT = [6250,6500,6750,7000]
        else:
            nT = [Tfin-500,Tfin-250,Tfin,Tfin+250,Tfin+500]

        if Gfin == 0.0:
            nG = [0.0,0.5,1.0,1.5]
        elif Gfin == 0.5:
            nG = [0.0,0.5,1.0,1.5]
        elif Gfin == 4.5:
            nG = [3.5,4.0,4.5,5.0]
        elif Gfin == 5.0:
            nG = [3.5,4.0,4.5,5.0]
        else:
            nG = [Gfin-1.0,Gfin-0.5,Gfin,Gfin+0.5,Gfin+1.0]
        calc = numpy.array(calculated)
        model_path2 = model_path+'vsini_'+str(rotfin)+'/'
        names = []

        for t in nT:
            for g in nG:
                nam = f'R_{rotfin}_{get_name(t,g,Zfin)}'
                I = numpy.where(calc == nam)[0]

                if len(I) == 0 and os.access(model_path2+nam,os.F_OK):
                    names.append(nam)

        for fits in names:
            calculated.append(fits)
            hd = _cached_getheader(model_path2+fits)
            T  = hd['TEFF']
            G  = hd['LOG_G']
            Z  = hd['FEH']

            sc = _cached_getdata(model_path2+fits)
            FF = sc[It]
            mwa = wat*(1+velfin/lux)

            NCCF = corr(L,F,mwa,FF,ons)

            vecZ.append(Z)
            vecT.append(T)
            vecG.append(G)
            vecR.append(rotfin)
            vecCF.append(NCCF)


        VZ = numpy.array(vecZ)
        VR = numpy.array(vecR)
        VT = numpy.array(vecT)
        VG = numpy.array(vecG)
        VF = numpy.array(vecCF)

        I = numpy.where((VZ == Zfin) & (VR == rotfin))[0]
        VT2 = VT[I]
        VG2 = VG[I]
        VF2 = VF[I]
        deltaT = 50.0
        deltaG = 0.05

        ejeT = numpy.arange(numpy.min(numpy.array(nT)),numpy.max(numpy.array(nT))+deltaT,deltaT)
        ejeG = numpy.arange(numpy.min(numpy.array(nG)),numpy.max(numpy.array(nG))+deltaG,deltaG)

        lejeT = len(ejeT)
        lejeG = len(ejeG)

        matCCF = numpy.zeros([lejeT,lejeG],float)

        for g in nG:
            pos = int(round((g-ejeG[0])/deltaG))
            I = numpy.where(VG2 == g)[0]
            if len(I) > 0:

                vTt,vCFt = orden(VT2[I],VF2[I])
                #print(vTt,vCFt)
                if len(I)>3:
                    tck  = interpolate.splrep(vTt,vCFt,k=3,s=0)
                    ynew = interpolate.splev(ejeT,tck,der=0)
                elif len(I) > 1:
                    tck  = interpolate.splrep(vTt,vCFt,k=len(I)-1,s=0)
                    ynew = interpolate.splev(ejeT,tck,der=0)
                else:
                    ynew = numpy.zeros(len(ejeT),float)+vCFt[0]

                matCCF[:,pos]=ynew

        for t in ejeT:
            pos1 = int(round((t-ejeT[0])/deltaT))
            y1 = matCCF[pos1,:]
            I = numpy.where(y1 != 0.0)[0]
            y1b = y1[I]
            x1b = ejeG[I]
            if len(I) > 0:
                if len(I)>3:
                    tck  = interpolate.splrep(x1b,y1b,k=3,s=0)
                    ynew = interpolate.splev(ejeG,tck,der=0)
                elif len(I) > 1:
                    tck  = interpolate.splrep(x1b,y1b,k=len(I)-1,s=0)
                    ynew = interpolate.splev(ejeG,tck,der=0)
                else:
                    ynew = numpy.zeros(len(ejeG),float)+y1b[0]
            matCCF[pos1,:]=ynew

        I = numpy.where(matCCF == numpy.max(matCCF))

        intert = ejeT[I[0]][0]
        interg = ejeG[I[1]][0]
        interz = Zfin
        interv = rotfin
        #print(intert,interg)
        maximCCF = numpy.max(matCCF)

    else:
        intert,interg,interz,interv = Tfin,Gfin,Zfin,rotfin
    #print('HI')
    if doplot:
        nam = f'R_{rotfin}_{get_name(Tfin,Gfin,Zfin)}'
        hd = _cached_getheader(model_path2+nam)
        sc = _cached_getdata(model_path2+nam)
        PWAV = ToVacuum(numpy.arange(len(sc))*hd['CD1_1']+hd['CRVAL1'])
        PWAV = PWAV*(1+velfin/lux)
        for i in range(L1.shape[0]):
            I = numpy.where((PWAV > L1[i,0]) & (PWAV < L1[i,-1]))[0]
            #print(L1[i])
            #print(F1[i])
            #print(i)
            f = plt.figure()
            plt.ylim(0.,1.01)
            plt.plot(L1[i],F1[i])
            plt.plot(PWAV[I],sc[I])
            plt.xlabel('wavelenth [A]')
            plt.ylabel('Continuum Normalized Flux')
            plt.savefig(f'{plot_dir}{plot_name}_{int(L1[i,0])}_{int(L1[i,-1])}.pdf', format='pdf')

    return [intert, interg, interz, interv, velfin,maximCCF]
